<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2013 IMIA net based solutions (d.frerich@imia.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

namespace IMIA\ImiaPageteaser\Controller;

use \TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * @package     imia_pageteaser
 * @subpackage  Controller
 * @author      David Frerich <d.frerich@imia.de>
 */
class TeaserController extends \IMIA\BaseExt\Controller\ActionController
{
    /**
     * @var integer
     */
    protected $limit;

    /**
     * @var string
     */
    protected $sessionDataKey;

    /**
     * @var array
     */
    protected $doktypes;

    /**
     * @var array
     */
    protected $categories;

    /**
     * @var array
     */
    protected $filter;

    /**
     * @var integer
     */
    protected $pageNum;

    /**
     * @var integer
     */
    protected $pageUid;

    /**
     * @var array
     */
    protected $contentData;

    /**
     * @var boolean
     */
    protected $allowParamOverwrite = false;

    /**
     * @var \IMIA\BaseExt\Utility\Session\Storage\SessionStorage
     */
    protected $sessionStorage;

    /**
     * @var \IMIA\ImiaPageteaser\Domain\Repository\PageRepository
     */
    protected $pageRepository;

    /**
     * @var \IMIA\ImiaPageteaser\Domain\Repository\CategoryRepository
     */
    protected $categoryRepository;

    /**
     * @param \IMIA\BaseExt\Utility\Session\Storage\SessionStorage $sessionStorage
     */
    public function injectSessionStorage(\IMIA\BaseExt\Utility\Session\Storage\SessionStorage $sessionStorage)
    {
        $this->sessionStorage = $sessionStorage;
    }

    /**
     * @param \IMIA\ImiaPageteaser\Domain\Repository\PageRepository $pageRepository
     */
    public function injectPageRepository(\IMIA\ImiaPageteaser\Domain\Repository\PageRepository $pageRepository)
    {
        $this->pageRepository = $pageRepository;
    }

    /**
     * @param \IMIA\ImiaPageteaser\Domain\Repository\CategoryRepository $categoryRepository
     */
    public function injectCategoryRepository(\IMIA\ImiaPageteaser\Domain\Repository\CategoryRepository $categoryRepository)
    {
        $this->categoryRepository = $categoryRepository;
    }

    public function indexAction()
    {
        if ($this->getLimit() > 0) {
            $result = $this->getResult();
            $this->view->assign('pages', $this->getPages($result));
            $this->view->assign('pageNum', $this->getPageNum());
            $this->view->assign('pageCount', $this->getPagesCount($result));
        }

        $this->view->assign('indexPage', $this->getPage());
        $this->view->assign('contentObject', $this->getContentData());
        $this->view->assign('filter', $this->getFilter());
        $this->view->assign('filterOptions', $this->getFilterOptions($this->getFirstPage(), $this->getLastPage()));
    }

    public function navigationAction()
    {
        $this->allowParamOverwrite = true;
        $this->updateSettings();

        $result = $this->getResult();
        $pageUid = $GLOBALS['TSFE']->id;
        $pageFound = false;
        $prev = null;
        $next = null;

        $backPid = (int)GeneralUtility::_GP('backPid');
        $overview = $backPid ?: $GLOBALS['TSFE']->page['pid'];

        foreach($result as $uid) {
            if ($pageFound) {
                $next = $uid;
                break;
            } elseif ($uid == $pageUid) {
                $pageFound = true;
            } else {
                $prev = $uid;
            }
        }

        $additionalParams = array();
        if ($backPid) {
            $additionalParams['backPid'] = $backPid;
        }
        if ($this->getContentUid() && $this->getContentUid() != $this->configurationManager->getContentObject()->data['uid']) {
            $additionalParams['contentUid'] = $this->getContentUid();
        }

        $this->view->assign('prev', $this->pageRepository->findOneByUid((int)$prev));
        $this->view->assign('next', $this->pageRepository->findOneByUid((int)$next));
        $this->view->assign('overview', $this->pageRepository->findOneByUid((int)$overview));
        $this->view->assign('additionalParams', $additionalParams);
    }

    /**
     * Overwrite settings, if alternative contentUid is defined
     */
    protected function updateSettings()
    {
        $contentData = $this->getContentData();
        if ($contentData['uid'] != $this->configurationManager->getContentObject()->data['uid']) {
            $flexFormService = $this->objectManager->get('TYPO3\CMS\Extbase\Service\FlexFormService');
            $flexFormData = $flexFormService->convertFlexFormContentToArray($contentData['pi_flexform']);
            if ($flexFormData && array_key_exists('settings', $flexFormData) && is_array($flexFormData['settings'])) {
                foreach ($flexFormData['settings'] as $key => $val) {
                    $this->settings[$key] = $val;
                }
            }
        }
    }

    /**
     * @return array
     */
    protected function getResult()
    {
        $resultKey = 'result_' . $GLOBALS['TSFE']->sys_language_uid;
        $modifiedKey = 'modified_' . $GLOBALS['TSFE']->sys_language_uid;

        $sesData = $this->sessionStorage->get($this->getSessionDataKey());
        if (!$sesData) {
            $sesData = array();
        }

        $result = null;
        if ($this->request->getMethod() == 'POST') {
            $sesData = array('filter' => $this->getFilter());
        } else {
            if (array_key_exists($resultKey, $sesData)) {
                $result = $sesData[$resultKey];
            }
        }

        $lastModified = $this->pageRepository->findLastModified();
        if (!array_key_exists($modifiedKey, $sesData) || $sesData[$modifiedKey] < $lastModified) {
            $sesData[$modifiedKey] = $lastModified;
            unset($result);
        }

        if (!$result) {
            switch ($this->getSetting('displayType')) {
                case 'selectedsubpages':
                    $result = $this->getSelectedSubPages();
                    break;
                case 'selectedpages':
                    $result = $this->getSelectedPages();
                    break;
                case 'allpages':
                    $result = $this->getAllPages();
                    break;
                case 'teaserpages':
                    $result = $this->getTeaserPages();
                    break;
                case 'selectedteaserpages':
                    $result = $this->getSelectedTeaserPages();
                    break;
                case 'subpages':
                default:
                    $result = $this->getSubPages();
                    break;
            }

            $sesData[$resultKey] = $result;
        }

        $this->sessionStorage->set($this->getSessionDataKey(), $sesData);

        return $result;
    }

    /**
     * @param array $result
     * @return \TYPO3\CMS\Extbase\Persistence\QueryResultInterface
     */
    protected function getPages($result)
    {
        return $this->pageRepository->findByUids(
            $result,
            $this->getSetting('date'),
            $this->getLimit(),
            $this->getOffset(),
            array($this->getSetting('sorting'), $this->getSetting('sortingDirection'))
        );
    }

    /**
     * @param array $result
     * @return integer
     */
    protected function getPagesCount($result)
    {
        $pagesCount = $this->pageRepository->countByUids(
            $result,
            $this->settings['date']
        );

        return ceil($pagesCount / $this->getLimit());
    }

    /**
     * @param string $orderDirection
     * @return \IMIA\ImiaPageteaser\Domain\Model\Page
     */
    protected function getFirstPage($orderDirection = 'asc')
    {
        switch ($this->getSetting('displayType')) {
            case 'selectedsubpages':
                $result = $this->getSelectedSubPages(true, $orderDirection);
                break;
            case 'selectedpages':
                $result = $this->getSelectedPages(true, $orderDirection);
                break;
            case 'allpages':
                $result = $this->getAllPages(true, $orderDirection);
                break;
            case 'teaserpages':
                $result = $this->getTeaserPages(true, $orderDirection);
                break;
            case 'selectedteaserpages':
                $result = $this->getSelectedTeaserPages(true, $orderDirection);
                break;
            case 'subpages':
            default:
                $result = $this->getSubPages(true, $orderDirection);
                break;
        }

        return $result;
    }

    /**
     * @return \IMIA\ImiaPageteaser\Domain\Model\Page
     */
    protected function getLastPage()
    {
        return $this->getFirstPage('desc');
    }

    /**
     * @return array
     */
    protected function getContentData()
    {
        if (!$this->contentData) {
            if ($this->allowParamOverwrite && GeneralUtility::_GP('contentUid')) {
                $this->contentData = array_shift($GLOBALS['TYPO3_DB']->exec_SELECTgetRows(
                    '*', 'tt_content', 'uid = ' . (int)GeneralUtility::_GP('contentUid')));
            } else {
                $this->contentData = $this->configurationManager->getContentObject()->data;
            }
        }

        return $this->contentData;
    }

    /**
     * @return integer
     */
    protected function getContentUid()
    {
        $contentData = $this->getContentData();

        return $contentData['uid'];
    }

    /**
     * @return integer
     */
    protected function getPage()
    {
        return $this->pageRepository->findOneByUid($this->getPageUid());
    }

    /**
     * @return integer
     */
    protected function getPageUid()
    {
        if (!$this->pageUid) {
            if ($this->allowParamOverwrite && GeneralUtility::_GP('backPid')) {
                $this->pageUid = (int)GeneralUtility::_GP('backPid');
            } else {
                $this->pageUid = $GLOBALS['TSFE']->id;
            }
        }

        return $this->pageUid;
    }

    /**
     * @param \IMIA\ImiaPageteaser\Domain\Model\Page $firstPage
     * @return array
     */
    protected function getFilterOptions($firstPage = null, $lastPage = null)
    {
        $filterOptions = array(
            'month'    => array(0 => $this->translate('filter_choose')),
            'year'     => array(0 => $this->translate('filter_choose')),
        );

        if ($firstPage && $lastPage) {
            for ($month = 1; $month <= 12; $month++) {
                $filterOptions['month'][$month] = $this->translate('filter_month_' . $month);
            }

            if ($firstPage->getLastUpdated()) {
                if ($this->getSetting('date') == 'past') {
                    for ($year = (int)$firstPage->getLastUpdated()->format('Y'); $year >= date('Y'); $year--) {
                        $filterOptions['year'][$year] = $year;
                    }
                } else {
                    for ($year = (int)$firstPage->getLastUpdated()->format('Y'); $year <= (int)$lastPage->getLastUpdated()->format('Y'); $year++) {
                        $filterOptions['year'][$year] = $year;
                    }
                }
            }
        }

        return $filterOptions;
    }

    /**
     * @param boolean $getFirst
     * @param string $orderDirection
     * @return mixed
     */
    protected function getSelectedSubPages($getFirst = false, $orderDirection = 'asc')
    {
        if (!$getFirst) {
            $result = $this->pageRepository->findUidsByPidsAndDoktypesAndCategories(
                array_map('intval', explode(',', $this->getSetting('pages'))),
                $this->getDoktypes(),
                $this->getCategories(),
                $this->getFilter(),
                array($this->getSetting('sorting'), $this->getRealOrderDirection($orderDirection))
            );
        } else {
            $result = array_shift($this->pageRepository->findByPidsAndDoktypesAndCategories(
                array_map('intval', explode(',', $this->getSetting('pages'))),
                $this->getDoktypes(),
                $this->getCategories(),
                null,
                1,
                0,
                array('lastUpdated', $orderDirection)
            ));
        }

        return $result;
    }

    /*
     * @param boolean $getFirst
     * @param string $orderDirection
     * @return mixed
     */
    protected function getSelectedPages($getFirst = false, $orderDirection = null)
    {
        if (!$getFirst) {
            $result = $this->pageRepository->findUidsByUidsAndDoktypesAndCategories(
                array_map('intval', explode(',', $this->getSetting('pages'))),
                $this->getDoktypes(),
                $this->getCategories(),
                $this->getFilter(),
                array($this->getSetting('sorting'), $this->getRealOrderDirection($orderDirection))
            );
        } else {
            $result = array_shift($this->pageRepository->findByUidsAndDoktypesAndCategories(
                array_map('intval', explode(',', $this->getSetting('pages'))),
                $this->getDoktypes(),
                $this->getCategories(),
                null,
                1,
                0,
                array('lastUpdated', $orderDirection)
            ));
        }

        return $result;
    }

    /**
     * @param boolean $getFirst
     * @param string $orderDirection
     * @return mixed
     */
    protected function getAllPages($getFirst = false, $orderDirection = 'asc')
    {
        if (!$getFirst) {
            $result = $this->pageRepository->findUidsByDoktypesAndCategories(
                $this->getDoktypes(),
                $this->getCategories(),
                $this->getFilter(),
                array($this->getSetting('sorting'), $this->getRealOrderDirection($orderDirection))
            );
        } else {
            $result = array_shift($this->pageRepository->findByDoktypesAndCategories(
                $this->getDoktypes(),
                $this->getCategories(),
                null,
                1,
                0,
                array('lastUpdated', $orderDirection)
            ));
        }

        return $result;
    }

    /**
     * @param boolean $getFirst
     * @param string $orderDirection
     * @return mixed
     */
    protected function getTeaserPages($getFirst = false, $orderDirection = 'asc')
    {
        if (!$getFirst) {
            $result = $this->pageRepository->findTeaserPagesByUids(
                array($this->getPageUid())
            );
        } else {
            $pageUids = $this->pageRepository->findTeaserPagesByUids(
                array_map('intval', explode(',', $this->getSetting('pages')))
            );

            if (strtolower($orderDirection) == 'desc') {
                $pageUid = array_pop($pageUids);
            } else {
                $pageUid = array_shift($pageUids);
            }

            $result = $this->pageRepository->findOneByUid($pageUid);
        }

        return $result;
    }

    /**
     * @param boolean $getFirst
     * @param string $orderDirection
     * @return mixed
     */
    protected function getSelectedTeaserPages($getFirst = false, $orderDirection = null)
    {
        if (!$getFirst) {
            $result = $this->pageRepository->findTeaserPagesByUids(
                array_map('intval', explode(',', $this->getSetting('pages')))
            );
        } else {
            $pageUids = $this->pageRepository->findTeaserPagesByUids(
                array_map('intval', explode(',', $this->getSetting('pages')))
            );

            if (strtolower($orderDirection) == 'desc') {
                $pageUid = array_pop($pageUids);
            } else {
                $pageUid = array_shift($pageUids);
            }

            $result = $this->pageRepository->findOneByUid($pageUid);
        }

        return $result;
    }

    /**
     * @param boolean $getFirst
     * @param string $orderDirection
     * @return mixed
     */
    protected function getSubPages($getFirst = false, $orderDirection = null)
    {
        if (!$getFirst) {
            $result = $this->pageRepository->findUidsByPidsAndDoktypesAndCategories(
                array($this->getPageUid()),
                $this->getDoktypes(),
                $this->getCategories(),
                $this->getFilter(),
                array($this->getSetting('sorting'), $this->getRealOrderDirection($orderDirection))
            );
        } else {
            $result = array_shift($this->pageRepository->findByPidsAndDoktypesAndCategories(
                array($this->getPageUid()),
                $this->getDoktypes(),
                $this->getCategories(),
                null,
                1,
                0,
                array('lastUpdated', strtolower($orderDirection))
            ));
        }

        return $result;
    }

    /**
     * @param string $orderDirection
     * @return string
     */
    protected function getRealOrderDirection($orderDirection)
    {
        if (strtolower($this->getSetting('sortingDirection')) == 'asc') {
            if (strtolower($orderDirection) == 'asc') {
                $realOrderDirection = 'asc';
            } else {
                $realOrderDirection = 'desc';
            }
        } else {
            if (strtolower($orderDirection) == 'asc') {
                $realOrderDirection = 'desc';
            } else {
                $realOrderDirection = 'asc';
            }
        }

        return $realOrderDirection;
    }

    /**
     * @return array
     */
    protected function getDoktypes()
    {
        if (!$this->doktypes) {
            $this->doktypes = array();
            foreach (explode(',', $this->getSetting('doktypes')) as $doktype) {
                $this->doktypes[] = (int)$doktype;

                switch ((int)$doktype) {
                    case 1:
                        $this->doktypes[] = 3;
                        $this->doktypes[] = 4;
                        break;
                    case 100:
                        $this->doktypes[] = 103;
                        $this->doktypes[] = 104;
                        break;
                    case 110:
                        $this->doktypes[] = 111;
                        $this->doktypes[] = 112;
                        $this->doktypes[] = 113;
                        $this->doktypes[] = 114;
                        break;
                    case 120:
                        $this->doktypes[] = 123;
                        $this->doktypes[] = 124;
                        break;
                    case 130:
                        $this->doktypes[] = 133;
                        $this->doktypes[] = 134;
                        break;
                    case 140:
                        $this->doktypes[] = 143;
                        $this->doktypes[] = 144;
                        break;
                }
            }
        }

        return $this->doktypes;
    }

    /**
     * @return array
     */
    protected function getFilter()
    {
        if (!$this->filter) {
            $this->filter = array();

            $sesData = $this->sessionStorage->get($this->getSessionDataKey());
            if (array_key_exists('filter', $sesData)) {
                $this->filter = $sesData['filter'];
            }

            if ($this->request->hasArgument('filter') && $this->request->getArgument('filter')) {
                $this->filter = (array)$this->request->getArgument('filter');

                $sesData['filter'] = $this->filter;
                $this->sessionStorage->set($this->getSessionDataKey(), $sesData);
            }
        }

        $filter = $this->filter;
        foreach ($this->getHookObjects() as $hook) {
            if (method_exists($hook, 'getFilter')) {
                $hook->getFilter($filter, $this->settings, $this);
            }
        }

        return $filter;
    }

    /**
     * @return integer
     */
    protected function getPageNum()
    {
        if (!$this->pageNum) {
            $this->pageNum = 1;
            if ($this->request->hasArgument('page') && $this->request->getArgument('page')) {
                $this->pageNum = (int)$this->request->getArgument('page');
            }
        }

        return $this->pageNum;
    }

    /**
     * @return integer
     */
    protected function getLimit()
    {
        if (!$this->limit) {
            $this->limit = (int)$this->getSetting('limit');

            foreach ($this->getHookObjects() as $hook) {
                if (method_exists($hook, 'getLimit')) {
                    $hook->getLimit($this->limit, $this->getFilter(), $this);
                }
            }
        }

        return $this->limit;
    }

    /**
     * @return string
     */
    protected function getSessionDataKey()
    {
        if (!$this->sessionDataKey) {
            $this->sessionDataKey = 'pageteaser_' . $this->getPageUid() . '_' . ($this->getContentUid() ?: md5(serialize($this->getContentData())));

            foreach ($this->getHookObjects() as $hook) {
                if (method_exists($hook, 'getSessionDataKey')) {
                    $hook->getSessionDataKey($this->sessionDataKey, $this->getPageUid(), $this->getContentData(), $this->settings, $this);
                }
            }
        }

        return $this->sessionDataKey;
    }

    /**
     * @return integer
     */
    protected function getOffset()
    {
        return $this->getPageNum() * $this->getLimit() - $this->getLimit();
    }

    /**
     * @return array
     */
    protected function getCategories()
    {
        if (!$this->categories && (int)$this->getSetting('categories') > 0) {
            $categories = array();
            /**
             * @FIXME very dirty and not performant, is there a better way?
             */
            $allCategories = $this->categoryRepository->findAll();
            foreach ($allCategories as $category) {
                $categoryCollection = $this->objectManager->create('TYPO3\CMS\Core\Category\Collection\CategoryCollection', 'tt_content');
                $categoryCollection->setIdentifier($category->getUid());
                $categoryCollection->loadContents();

                $contentElements = $categoryCollection->getItems();
                foreach ($contentElements as $contentElement) {
                    if ($contentElement['uid'] == $this->getContentUid() || !$this->getContentUid()) {
                        $categories[] = $category->getUid();
                        foreach ($category->getAllChildren() as $childCategory) {
                            $categories[] = $childCategory->getUid();
                        }
                    }
                }
            }

            $this->categories = array_unique($categories);
        }

        return $this->categories;
    }
}